@@ -15,6 +15,9 @@
 # else
 #  define PTR_FMT "0x%08x"
 # endif
+// Solaris libc omits the leading 0x.
+#elif defined(__sun__) && defined(__svr4__)
+# define PTR_FMT "0x%p"
 #else
 # define PTR_FMT "%p"
 #endif
