@@ -5,6 +5,9 @@
    Regression test for a bug in msan/glibc integration,
    see https://sourceware.org/bugzilla/show_bug.cgi?id=16291
    and https://github.com/google/sanitizers/issues/547
+
+   XFAIL: FreeBSD
+   UNSUPPORTED: powerpc
 */
 
 #ifndef BUILD_SO
